# -*- coding: utf-8 -*-
# Python 3

"""
Dieses Skript extrahiert für jede Instanz (also jede Frage egal ob rhetorisch oder nicht) aus dem Switchboard Korpus einen entsprechenden Outputvektor.
Der Outputvektor für eine Instanz sieht folgendermaßen aus:
Frage   SatzDavor   SatzDanach  POS-Frage   POS-SatzDavor   POS-SatzDanach  Sprecherwechsel Label
Der Sprecherwechsel kann z.B. folgendermaßen aussehen: "ABA", was entspricht:
Im Satz davor spricht Sprecher A, die Frage wird von Sprecher B gestellt und im Satz danach spricht wieder Sprecher A.
Das Label ist 1 für "rhetorisch" (qh) und 2 für "nicht-rhetorisch" (alle anderen Fragetags).

Am Ende haben wir einen Vektor für jede Instanz, mit dem wir dann an die Featureextraktion gehen können, 
nach welcher wir dann am Ende die Vektoren für unseren Klassifizierer haben, die nur noch aus counts und binären Features bestehen.

n-gramme, POS-n-gramme, binäre Features für den Sprecherwechsel und mehr lassen sich sehr einfach aus dem Vektor generieren, den dieses Skript erzeugt.
"""

import re
import spacy

parser = spacy.load("en")

# Diese Funktion soll die Text-Parts für den Outputvektor bereinigen, sodass wir sinnvolle POS-Tags erhalten.

def cleanText(text):
    
    text = text.replace("/", "")    # entfernt end-of-line slashes
    text = text.replace("}", "")
    text = text.replace("{A", "")
    text = text.replace("{B", "")
    text = text.replace("{C", "")
    text = text.replace("{D", "")
    text = text.replace("{E", "")
    text = text.replace("{F", "")
    text = text.replace("{", "")
    text = text.replace("(( ", "")
    text = text.replace(" ))", "")
    text = text.replace("#", "")
    text = text.replace("--", "")
    text = re.sub("<.*>", "", text)     # entfernt <laughter>, <breathing>, usw.
    text = re.sub("\[ \[ \[ \[ \[.*\+(.*)\]", "\\1", text)
    text = re.sub("\[ \[ \[ \[.*\+(.*)\]", "\\1", text)
    text = re.sub("\[ \[ \[ .*\+(.*)\]", "\\1", text)
    text = re.sub("\[ \[ .*\+(.*)\]", "\\1", text)
    text = re.sub("\[ .*\+(.*)\]", "\\1", text)
    text = text.replace("+", "")
    text = text.replace("[", "")    # falls noch irgendwo Klammern übriggeblieben sind
    text = text.replace("]", "")    # falls noch irgendwo Klammern übriggeblieben sind
    text = re.sub("\s+", " ", text)
    
    return text
    

def filterParts(file):
    
    with open(file, "r") as f:
        content = f.read()
    
    listOfInstances = content.split("----------\n")
    
    listOfOutputInstances = []
    
    for instance in listOfInstances:
        
        lines = instance.split("\n")
        
        try:
            m = re.search("([^ \n]+)[ ]+.?.?(A|B)\.\d+ utt\d+:[ ]+(.+)", lines[0])        
            
            n = re.search("([^ \n]+)[ ]+.?.?(A|B)\.\d+ utt\d+:[ ]+(.+)", lines[1])
            
            o = re.search("([^ \n]+)[ ]+.?.?(A|B)\.\d+ utt\d+:[ ]+(.+)", lines[2])
            
            question = cleanText(n.group(3)).strip(" ")
            before = ""
            after = ""
            speakerOrder = ""
            
            if m:
                before += cleanText(m.group(3)).strip(" ")
                
                speakerOrder += m.group(2)
            
            speakerOrder += n.group(2)
            
            if o:
                after += cleanText(o.group(3)).strip(" ")
                speakerOrder += o.group(2)
            
            tag = n.group(1)
            
            if tag == "qh":
                tag = 1
                
            else:
                tag = 0
            
            outputInstance = question + "\t" + before + "\t" + after + "\t" + " ".join(x.tag_ for x in parser(question))+ "\t" + " ".join(x.tag_ for x in parser(before))+ "\t" + " ".join(x.tag_ for x in parser(after))+ "\t" + speakerOrder + "\t" + str(tag) + "\n"
            
            listOfOutputInstances.append(outputInstance)
            
        except IndexError:
            break
        
    with open("preparedInstances.txt", "w") as g:
        g.writelines(listOfOutputInstances)
    

if __name__ == "__main__":
    filterParts("allQuestionsContext.txt")
